# -*- coding: utf8 -*-
"""
Connection Settings
Created on 02/26/2016
@description:	Used for 
@author: 		Wen Gu
@contact: 		emptyset110@gmail.com
"""
import time
"""
雪球
"""
HEADERS_XUEQIU = {
	"Accept"			:	"application/json, text/javascript, */*; q=0.01"
,	"Accept-Encoding"	:	"gzip, deflate, sdch"
,	"Accept-Language"	:	"en-US,en;q=0.8"
,	"cache-control"		:	"no-cache"
,	"Connection"		:	"keep-alive"
,	"Host"				:	"xueqiu.com"
,	"Referer"			:	"https://xueqiu.com/hq"
,	"User-Agent"		:	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/49.0.2623.110 Safari/537.36"
,	"Upgrade-Insecure-Requests"	: "1"
,	"X-Requested-With"	:	"XMLHttpRequest"
}

HEADERS_XUEQIU_INDEX = {
	"Accept"			:	"text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8"
,	"Accept-Encoding"	:	"gzip, deflate, sdch"
,	"Accept-Language"	:	"en-US,en;q=0.8"
,	"Connection"		:	"keep-alive"
,	"Host"				:	"xueqiu.com"
,	"Upgrade-Insecure-Requests"	:	"1"
,	"User-Agent"		:	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/49.0.2623.110 Safari/537.36"
}


URL_XUEQIU_HQ = "https://xueqiu.com/hq"
URL_XUEQIU_QUOTE_ORDER = lambda page,columns,stockType : "https://xueqiu.com/stock/quote_order.json?page=%s&size=90&order=desc&exchange=CN&stockType=%s&column=%s&orderBy=symbol&_=%s"	% ( page,stockType, columns,int(time.time()*1000) )
CONST_XUEQIU_QUOTE_ORDER_COLUMN = "symbol,name,current,chg,percent,last_close,open,high,low,volume,amount,market_capital,pe_ttm,high52w,low52w,hasexist"

# 用于获取k线
# fqType: before,normal,after,
# start,end: 13位时间戳
# period = 1day(1d),5day(5d),1week,1month
URL_XUEQIU_KLINE = lambda symbol,period,fqType,begin,end: "https://xueqiu.com/stock/forchartk/stocklist.json?symbol=%s&period=%s&type=%s&begin=%s&end=%s&_=%s" % ( symbol,period,fqType,begin,end,int(time.time()*1000) )
URL_XUEQIU_CHART = lambda symbol,period: "https://xueqiu.com/stock/forchart/stocklist.json?symbol=%s&period=%s&one_min=1&_%s" % ( symbol,period,int(time.time()*1000) )

# 用于获取基本面或者实时quote
# code可以取多个，用逗号分割
URL_XUEQIU_QUOTE = lambda symbols : "http://xueqiu.com/v4/stock/quote.json?code=%s&_=%s" % (symbols, int(time.time()*1000) )
CONST_XUEQIU_QUOTE_COLUMN = "symbol,exchange,code,name,current,percentage,change,open,high,low,close,last_close,high52week,low52week,volume,volumeAverage,marketCapital,eps,pe_ttm,pe_lyr_beta,totalShares,time,afterHours,afterHoursPct,afterHoursChg,updateAt,dividednd,yield,turnover_rate,instOwn,rise_stop,fall_stop,currency_unit,amount,net_assets,hasexist,has_warrant,type,flag,rest_day,amplitude,lot_size,tick_size,kzz_stock_symbol,kzz_stock_name,kzz_stock_current,kzz_convert_price,kzz_convert_value"
{	"SZ000001":
		{	
			"symbol"			:	"SZ000001"			# 代码
		,	"exchange"			:	"SZ"				# SZ/SH
		,	"code"				:	"000001"			# code
		,	"name"				:	"平安银行"			# 中文名	
		,	"current"			:	"10.59"				# 当前价
		,	"percentage"		:	"-1.21"				# 涨幅
		,	"change"			:	"-0.13"				# 与开盘价差值
		,	"open"				:	"10.72"				# 开盘价
		,	"high"				:	"10.73"				# 最高价
		,	"low"				:	"10.59"				# 最低价
		,	"close"				:	"8.27"				# 今日收盘价
		,	"last_close"		:	"10.72"				# 昨收盘价
		,	"high52week"		:	"19.8"				# 52周最高价
		,	"low52week"			:	"9.3"				# 52周最低价
		,	"volume"			:	"3.7771959E7"		# 成交量
		,	"volumeAverage"		:	"86076925"			# 
		,	"marketCapital"		:	"1.5152888031201E11"#
		,	"eps"				:	"1.56"				# 每股收益
		,	"pe_ttm"			:	"6.9302"			# 市盈率TTM
		,	"pe_lyr"			:	"6.9302"			# 市盈率LYR
		,	"beta"				:	"0.0"				# beta值		
		,	"totalShares"		:	"14308676139"		# 总股本
		,	"time"				:	"Thu Apr 07 15:14:55 +0800 2016"	# 时间
		,	"afterHours"		:	"0.0"
		,	"afterHoursPct"		:	"0.0"
		,	"afterHoursChg"		:	"0.0"
		,	"updateAt"			:	"1459944006501"		# 更新时间戳
		,	"dividend"			:	"0.174"				# 每股股息
		,	"yield"				:	"1.31"				# 振幅
		,	"turnover_rate"		:	"0.32"				# 换手率
		,	"instOwn"			:	"0.0"
		,	"rise_stop"			:	"11.79"				# 涨停
		,	"fall_stop"			:	"9.65"				# 跌停
		,	"currency_unit"		:	"CNY"				# 货币
		,	"amount"			:	"4.0194905057E8"	# 成交额
		,	"net_assets"		:	"11.2869"			# 
		,	"hasexist"			:	""
		,	"has_warrant"		:	"0"
		,	"type"				:	"11"				# 11=深A，12=沪A
		,	"flag"				:	"1"
		,	"rest_day"			:	""
		,	"amplitude"			:	"1.31"
		,	"lot_size"			:	"100"
		,	"min_order_quantity":	"0"
		,	"max_order_quantity":	"0"
		,	"tick_size"			:	"0.01"
		,	"kzz_stock_symbol"	:	""
		,	"kzz_stock_name"	:	""
		,	"kzz_stock_current"	:	"0.0"
		,	"kzz_convert_price"	:	"0.0"
		,	"kzz_covert_value"	:	"0.0"
		,	"kzz_cpr"			:	"0.0"
		,	"kzz_putback_price"	:	"0.0"
		,	"kzz_convert_time"	:	""
		,	"kzz_redempt_price"	:	"0.0"
		,	"kzz_straight_price":	"0.0"
		,	"kzz_stock_percent"	:	""
		,	"pb"				:	"0.94"
		,	"benefit_before_tax":	"0.0"
		,	"benefit_after_tax"	:	"0.0"
		,	"convert_bond_ratio":	""
		,	"totalissuescale"	:	""
		,	"outstandingamt"	:	""
		,	"maturitydate"		:	""
		,	"remain_year"		:	""
		,	"convertrate"		:	"0.0"
		,	"interestrtmemo"	:	""
		,	"release_date"		:	""
		,	"circulation"		:	"0.0"
		,	"par_value"			:	"0.0"
		,	"due_time"			:	"0.0"
		,	"value_date"		:	""
		,	"due_date"			:	""
		,	"publisher"			:	""
		,	"redeem_type"		:	"T"
		,	"issue_type"		:	""
		,	"bond_type"			:	""
		,	"warrant"			:	""
		,	"sale_rrg"			:	""
		,	"rate"				:	""
		,	"after_hour_vol"	:	"0"
		,	"float_shares"		:	"11804054528"	# 流通股数
		,	"float_market_capital":	"1.2493451188647E11"
		,	"disnext_pay_date"	:	""
		,	"convert_rate"		:	"0.0"
		,	"psr"				:	"1.5758"
		}
}


# 实时盘口
URL_XUEQIU_QUOTEC = lambda symbol : "http://xueqiu.com/v4/stock/quotec.json?code=%s&_=%s" % (symbol, int(time.time()*1000) )
URL_XUEQIU_PANKOU = lambda symbol : "https://xueqiu.com/stock/pankou.json?symbol=%s&_=%s" % (symbol, int(time.time()*1000) )
{	
	"symbol"	:	"SZ300061"
,	"time"		:	"Mar 18, 2016 10:38:12 AM"
,	"bp1"		:	21.4	,	"bc1"		:	15112
,	"bp2"		:	21.36	,	"bc2"		:	20
,	"bp3"		:	21.35	,	"bc3"		:	122
,	"bp4"		:	21.34	,	"bc4"		:	1
,	"bp5"		:	21.33	,	"bc5"		:	1
,	"bp6"		:	0.0		,	"bc6"		:	0
,	"bp7"		:	0.0		,	"bc7"		:	0
,	"bp8"		:	0.0		,	"bc8"		:	0
,	"bp9"		:	0.0		,	"bc9"		:	0
,	"bp10"		:	0.0		,	"bc10"		:	0
,	"current"	:	21.4	,	"sp1"		:	0.0
,	"sc1"		:	0		,	"sp2"		:	0.0
,	"sc2"		:	0		,	"sp3"		:	0.0
,	"sc3"		:	0		,	"sp4"		:	0.0
,	"sc4"		:	0		,	"sp5"		:	0.0
,	"sc5"		:	0		,	"sp6"		:	0.0
,	"sc6"		:	0		,	"sp7"		:	0.0
,	"sc7"		:	0		,	"sp8"		:	0.0
,	"sc8"		:	0		,	"sp9"		:	0.0
,	"sc9"		:	0		,	"sp10"		:	0.0
,	"sc10"		:	0		,	"buypct"	:	100.0
,	"sellpct"	:	0.0		,	"diff"		:	15256	
,	"ratio"		:	100.0
}



# -*- coding: utf8 -*-
"""
# Created on 05/06/2016
# @author: Emptyset
# @contact: emptyset110@gmail
"""
# 以下是自动生成的 #
# --- 导入系统配置
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import dHydra.core.util as util
from dHydra.core.Action import Action
from dHydra.core.Event import Event
from dHydra.config import connection as CON
from dHydra.config import const as C
from dHydra.core.Functions import *
# --- 导入自定义配置
import socket
import time
import threading

# 以上是自动生成的 #
class SinaL2TCPAction(Action):
	def __init__(	self
				,	name
				,	producerList = [
					{	
						"name"	:	"SinaLevel2WS"
					,	"pName"	:	"SinaL2TCP.quotation"
					,	"query"	:	['quotation']	#只获取10档行情
					} 
					]
				,	addr = "127.0.0.1"
				,	port 	= 9999
				,	interval = 0.2
				, 	**kwargs
				):
		# 用户自定义自动加载的_producerList
		self._producerList = producerList
		# 设置进程检查消息队列的间隔
		self._interval = interval
		super().__init__(name, **kwargs)
		self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		while True:
			try:
				self.s.connect( ( addr, port ) )
				break
			except Exception as e:
				self.logger.error( "{}:{}建立socket连接失败, {}\n5秒后重试".format(addr,port,e) )
				time.sleep(5)
		self.mutex = threading.Lock()

	# 需要重写的方法
	def handler(self):
		while not self._queue.empty():
			self.mutex.acquire()
			event = self._queue.get(True)
			self.mutex.release()
			if isinstance(event.data, list):
				for data in event.data:
					try:
						self.s.send( event.data.encode(encoding="utf-8") )
					except Exception as e:
						self.logger.error( "{}".format(e) )
						time.sleep(2)
			else:
				try:
					print( "发送{}".format(event.data) )
					self.s.send( event.data.encode(encoding="utf-8") )
				except Exception as e:
					self.logger.error( "{}".format(e) )
					time.sleep(2)



# -*- coding: utf8 -*-

import logging
from dHydra.core.Functions import *

class Vendor:
	def __init__(self, logLevel = logging.INFO):
		self.logger = self.get_logger()
		pass

	def get_logger(self):
		logger = logging.getLogger(self.__class__.__name__)
		return logger


# -*- coding: utf8 -*-
"""
为了规范推送数据的格式
推荐用Event对象进行数据推送
通常需要设置几个参数：
	eventType指这个事件的类型：
		由于action从不同的producer获取Event，action需要被告知这个event是什么类型的
	dataType是指data的数据类型：
		例如dataframe，json等等
		同一个producer很可能产生不同数据类型的数据，因此，需要用dataType来进行注明
	data： 
		需要传递的数据主体
	timestamp： 
		数据产生的时间戳

"""
class Event:

	def __init__(
					self
				,	eventType = None
				,	dataType = None
				,	timestamp = None
				,	data = None):
		self.eventType = eventType
		self.dataType = dataType
		self.timestamp = timestamp
		self.data = data


# -*- coding: utf8 -*-
"""
dHydra框架的全局方法，在主程序运行时会被引用
---
Created on 03/27/2016
@author: Wen Gu
@contact: emptyset110@gmail.com
"""
import importlib
import sys
import json
import hashlib
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import os
import logging

# print("加载：Functions.py")


"""
V方法，动态加载数据API类
V可以用Vendor来记忆
"""
def V(name, vName = None):
	logger = logging.getLogger('Functions')
	if vName is None:
		vName = "V-"+name
	className = name + 'Vendor'
	name = 'vendor.' + name + '.' + className
	try:
		instance = getattr( __import__(name, globals(),locals(),[className], 0), className )()
	except ImportError:
		try:
			instance = getattr( __import__("dHydra."+name, globals(),locals(),[className], 0), className )()
		except Exception as e:
			logger.critical(e)
			pass
	except Exception as e:
		logger.critical(e)
		pass

	return instance


"""
P方法，动态获取生产者类
P可以用Producer来记忆
"""
def P(name, pName, **kwargs):
	logger = logging.getLogger('Functions')
	if pName is None:
		print("pName参数不允许为空，请给producer实例设置一个名字")
		print("命名规范：<actionName.producerName>")
		return False
	# 将参数排序来保证唯一性
	json_kwargs = json.dumps( sorted(kwargs.items()) )
	producerHash = hashlib.sha1( ('name'+json_kwargs).encode('utf8') ).hexdigest()

	if producerHash in PRODUCER_HASH:
		# 这个Instance已经存在
		PRODUCER_NAME[pName] = PRODUCER_HASH[producerHash]
		return PRODUCER_HASH[producerHash]
	else:
		className = name + 'Producer'
		name = 'producer.' + name + '.' + className
		try:
			instance = getattr( __import__(name, globals(),locals(),[className], 0), className )(name=pName, **kwargs)
		except ImportError:
			try:
				instance = getattr( __import__("dHydra."+name, globals(),locals(),[className], 0), className )(name=pName, **kwargs)
			except Exception as e:
				logger.critical(e)
				exit()
		except Exception as e:
			logger.critical(e)
			exit()

		# print(instance)
		PRODUCER_NAME[pName] = instance
		PRODUCER_HASH[producerHash] = instance
		logger.info("生成Producer:\t{}\t{}".format(pName,className) )
		return instance

"""
A方法，动态获取Action类
A可以用Action来记忆
"""
def A(name, aName = None, **kwargs):
	logger = logging.getLogger('Functions')
	if aName is None:
		aName = "A-" + name
	className = name + 'Action'
	name = 'action.' + name + '.' + className
	try:
		return getattr( __import__(name, globals(),locals(),[className], 0), className )(name=aName, **kwargs)
	except ImportError:
		try:
			return getattr( __import__("dHydra." + name, globals(),locals(),[className], 0), className )(name=aName, **kwargs)
		except Exception as e:
			logger.critical(e)
			print(e)
			pass

"""
根据pName或者hash获取已经生成的Producer实例
"""
def get_producer(pName = None, pHash = None):
	logger = logging.getLogger('Functions')
	if ( pName is None and pHash is None ):
		logger.error("错误：pName和pHash两个参数中至少要有一个不为空")
		return False
	try:
		return PRODUCER_NAME[pName]
	except:
		try:
			return PRODUCER_HASH[pHash]
		except:
			logger.error("没有找到对应的Producer")
			return False
		


# -*- coding: utf8 -*-
"""
全局变量在这里定义
"""
import traceback
import sys
import os
import json
# print("加载: Globals.py")

# producer
PRODUCER_HASH = dict()
PRODUCER_NAME = dict()

actionDict = dict()


# 读取配置
f_config = open( os.getcwd() + "/config.json" )
config = json.load( f_config )


# -*- coding: utf8 -*-
from dHydra.core.Action import Action
from dHydra.core import util
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
from dHydra.core.Functions import *

import re
from datetime import datetime
import threading

class PrintSinaL2Action(Action):
	def __init__(self, name, **kwargs):
		# 用户自定义自动加载的_producerList
		self._producerList = [
			{	
				"name"	:	"SinaLevel2WS"
			,	"pName"	:	"PrintSinaL2.SinaLevel2-quotation"
			,	"query"	:	['quotation']	#只获取10档行情
			}
		# ,	{
		# 		"name"	:	"SinaLevel2WS"
		# 	,	"pName"	:	"PrintSinaL2.SinaLevel2-deal"
		# 	,	"query"	:	['deal']
		# 	}
		]
		# 设置进程检查消息队列的间隔
		self._interval = 0.1
		super().__init__(name, **kwargs)
		self.logger.info(self._name +"初始化")
		self.count = 0
 
	# 需要重写的方法
	def handler(self):
		while not self._queue.empty():
			dt = datetime.now()
			event = self._queue.get(True)
			# event.data = self.ws_parse(message = event.data)
			if isinstance(event.data, list):
				for data in event.data:
					print("PrintSinaL2:\n", data )

					# print(self.count)
					pass
			else:
				print( datetime.now() )
				print(event.data)
				
		# self.logger.info("线程退出")

	"""
	用于解析Sina l2的函数
	"""
	def ws_parse(self, message):
		dataList = re.findall(r'(?:((?:2cn_)?((?:sh|sz)[\d]{6})(?:_0|_1|_orders|_i)?)(?:=)(.*)(?:\n))',message)
		result = list()
		for data in dataList:
			if (len(data[0])==12):	# quotation
				wstype = 'quotation'
				self.count += 1
			elif ( (data[0][-2:]=='_0') | (data[0][-2:]=='_1') ):
				wstype = 'deal'
			elif ( data[0][-6:]=='orders' ):
				wstype = 'orders'
			elif ( (data[0][-2:]=='_i') ):
				wstype = 'info'
			else:
				wstype = 'other'
			result = self.ws_parse_to_list(wstype=wstype,symbol=data[1],data=data[2],result=result)
		return result

	def ws_parse_to_list(self,wstype,symbol,data,result):
		data = data.split(',')
		if (wstype == 'deal'):
			for d in data:
				x = list()
				x.append(wstype)
				x.append(symbol)
				x.extend( d.split('|') )
				result.append(x)
		else:
			x = list()
			x.append(wstype)
			x.append(symbol)
			x.extend(data)
			result.append(x)
		return result


# -*- coding: utf8 -*-
"""
雪球社区接口类 
Created on 03/17/2016
@author: Wen Gu
@contact: emptyset110@gmail.com
"""
# 以下是自动生成的 #
# --- 导入系统配置
import dHydra.core.util as util
from dHydra.core.Vendor import Vendor
from dHydra.core.Functions import V
from dHydra.config import connection as CON
from dHydra.config import const as C
# --- 导入自定义配置
from .connection import *
from .const import *
from .config import *
# 以上是自动生成的 #
import pytz
import requests
import asyncio
from pandas import DataFrame
import pandas
from datetime import datetime, timedelta
import functools
import threading
import pymongo
import time

class XueqiuVendor(Vendor):

	def __init__(self):
		super().__init__()
		self.session = requests.Session()
		# 先爬取一遍雪球页面，获取cookies
		xq = self.session.get(
			"https://xueqiu.com/hq"
		,	headers = HEADERS_XUEQIU
		)
		self.mongodb = None

	"""
	stockTypeList 	: list
		['sha','shb','sza','szb']分别代表沪A，沪B，深A，深B。如果为空则代表获取所有沪深AB股
		e.g: stockTypeList = ['sha','shb']即获取所有沪A沪B
	columns 		:	string
		默认为："symbol,name,current,chg,percent,last_close,open,high,low,volume,amount,market_capital,pe_ttm,high52w,low52w,hasexist"
	"""
	def get_stocks(	self
				,	stockTypeList	=	['sha','shb','sza','szb']
				,	columns 		=	CONST_XUEQIU_QUOTE_ORDER_COLUMN
		):
		for stockType in stockTypeList:
			print( "正在从雪球获取：{}".format( EX_NAME[stockType] ) )
			page = 1
			while True:
				response = self.session.get(
					URL_XUEQIU_QUOTE_ORDER(page,columns,stockType)
				,	headers = HEADERS_XUEQIU
				).json()
				df = DataFrame.from_records(response["data"], columns=response["column"])
				if 'stocks' not in locals().keys():
					stocks = df
				else:
					stocks = stocks.append(df)
				if df.size==0:
					break
				page += 1
		return stocks

	"""
	返回：set
	"""
	def get_symbols(	self
					,	stockTypeList = ['sha','shb','sza','szb']
		):
		symbols = self.get_stocks(stockTypeList = stockTypeList, columns = 'symbol')
		return set(symbols.symbol)

	"""
	获取雪球行情/基本面的接口
	"""
	def get_quotation(self, symbol=None, symbolSet=None, dataframe = True, threadNum = 3):
		if 'quotation' in self.__dict__.keys():
			del(self.quotation)
			# Cut symbolList
		symbolList = list(symbolSet)
		threads = []
		symbolListSlice = util.slice_list(num = threadNum,dataList = symbolList)
		for symbolList in symbolListSlice:
			loop = asyncio.new_event_loop()
			symbolsList = util.slice_list(step=50, dataList = symbolList)
			tasks = [self.get_quotation_task(symbols=symbols) for symbols in symbolsList]
			t = threading.Thread(target = util.thread_loop,args=(loop,tasks) )
			threads.append(t)
		for t in threads:
			t.start()
		for t in threads:
			t.join()

		if dataframe:
			self.quotation = DataFrame.from_records( self.quotation ).T
		return(self.quotation)


	@asyncio.coroutine
	def get_quotation_task(self, symbols):
		symbols = util.symbols_to_string(symbols)
		quotation = yield from self.fetch_quotation_coroutine(symbols = symbols)
		if 'quotation' not in self.__dict__.keys():
			self.quotation = quotation
		else:
			self.quotation.update( quotation )

	"""
	雪球单股基本面数据获取coroutine
	"""
	@asyncio.coroutine
	def fetch_quotation_coroutine(self, symbols=None):
		loop = asyncio.get_event_loop()
		if symbols is not None:
			async_req = loop.run_in_executor(None, functools.partial( self.session.get
			,	URL_XUEQIU_QUOTE(symbols)
			,	headers = HEADERS_XUEQIU
			) )
			try:
				quotation = yield from async_req
			except Exception as e:
				print(e)
				async_req = loop.run_in_executor(None, functools.partial( self.session.get
				,	URL_XUEQIU_QUOTE(symbols)
				,	headers = HEADERS_XUEQIU
				) )
				quotation = yield from async_req
			quotation = quotation.json()
		return(quotation)

	# """
	# 雪球单股基本面数据获取
	# 默认返回值格式是dict，若参数dataframe为True则返回dataframe
	# """
	# def fetch_quotation(self, symbols = None, dataframe = False):
	# 	symbols = util.symbols_to_string(symbols)
	# 	if symbols is not None:
	# 		quotation = self.session.get(
	# 			URL_XUEQIU_QUOTE(symbols)
	# 		,	headers = HEADERS_XUEQIU
	# 		).json()
	# 	if dataframe:
	# 		quotation = DataFrame.from_records( quotation ).T
	# 	return(quotation)

	"""
	雪球历史k线接口，包括前/后复权(默认不复权)
		period: 1day,1week,1month
	"""
	def get_kline(self, symbol, period = '1day', fqType = 'normal', begin = None, end = None, dataframe = True):
		if end is None:
			end = util.time_now()
		if isinstance(begin, str):
			begin = util.date_to_timestamp( begin )
		if isinstance(end, str):
			end = util.date_to_timestamp( end )
		try:
			response = self.session.get(
					URL_XUEQIU_KLINE( symbol = symbol, period = period, fqType = fqType, begin = begin, end = end )
				,	headers = HEADERS_XUEQIU
				,	timeout = 3
				)
			kline = response.json()
			time.sleep(0.5)
		except Exception as e:
			self.logger.warning("{}".format(e))
			self.logger.info(response.text)
			time.sleep(3)
			return None

		if kline["success"]=='true':
			if dataframe:
				if kline["chartlist"] is not None:
					df = DataFrame.from_records( kline["chartlist"] )
					df["time"] = pandas.to_datetime( df["time"] )
					df["time"] += timedelta(hours=8)
					df["symbol"] = symbol
					return df
				else:
					return DataFrame()
			else:
				return kline["chartlist"]
		else:
			return None

	"""
	将单股票历史k线存入mongodb
	"""
	def kline_to_mongodb(self, symbol, types=["normal","before","after"], end = None, dbName = 'stock', collectionName = 'kline_history', host='localhost', port=27017):
		types = ["normal","before","after"]
		if end is None:
			end = datetime.now().date()
		else:
			end = util.string_to_date(end)
		if self.mongodb is None:
			self.mongodb = V("DB").get_mongodb(host=host, port=port)
		if self.mongodb == False:
			self.logger.error("没有连接上mongodb")
			return False

		for fqType in types:
			# 先找到mongodb中这条股票的最新记录
			latest = self.mongodb[dbName][collectionName].find_one(
				{ "symbol"	:	symbol, "type" : fqType }
			,	sort = [("time", -1)]
			)
			if latest is not None:
				begin = ( latest["time"]+timedelta(days=1) ).strftime("%Y-%m-%d")
				self.logger.info("symbol = {}, {}\t最近更新记录为 {}".format(symbol,fqType,latest["time"]))
				if latest["time"].date() >= end:
					self.logger.info("不需要更新")
					return True
			else:
				begin = None
				self.logger.info("symbol = {}, {}\t无最近更新记录".format(symbol,fqType))

			self.logger.info("开始更新symbol = {} \t {}".format(symbol, fqType))
			kline = None
			while kline is None:
				kline = self.get_kline(symbol, begin = begin)

			if len(kline)>0:
				kline["type"] = fqType
				kline = kline.iloc[0:len(kline)].to_dict(orient="records")
				self.mongodb[dbName][collectionName].insert_many( kline )
		return True

	def kline_history(self, symbols = None,end = None, types = ["normal","before","after"], dbName = "stock", collectionName = "kline_history", host="localhost", port=27017):
		if symbols is None:
			# 我选择从新浪获取一份股票列表
			sina = V("Sina")
			symbolList = sina.get_symbols()
		elif isinstance(symbols, str):
			symbolList = symbols.split(',')
		else:
			symbolList = list(symbols)

		for symbol in symbolList:
			self.kline_to_mongodb(symbol, types=types,end = end, dbName=dbName, collectionName=collectionName, host=host, port=port)
		return True

	"""
	period  = '1d'  	只显示当日分钟线
			= '5d'		5分钟线，250行（最多5个交易日）
			= 'all'		历史周线
	"""
	def get_today(self, symbol, period = '1day', dataframe = True):
		quotation = self.session.get(
				URL_XUEQIU_CHART( symbol = symbol, period = period)
			,	headers = HEADERS_XUEQIU
			).json()
		if quotation["success"] == "true":
			if dataframe:
				df = DataFrame.from_records( quotation["chartlist"] )
				df["time"] = pandas.to_datetime( df["time"] )
				df["time"] += timedelta(hours=8)
				df["symbol"] = symbol
				return df
			else:
				return quotation["chartlist"]
		else:
			return False

	"""
	雪球键盘助手
	"""
	def keyboard_helper(self,symbol):
		response = self.session.get(
			"https://xueqiu.com/stock/search.json?code=%s&size=10&_=%s"%(symbol,int(t.time()*1000))
		).json()["stocks"]
		return response


# -*- coding: utf8 -*-
# 以下是自动生成的 #
# --- 导入系统配置
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import dHydra.core.util as util
from dHydra.core.Producer import Producer
from dHydra.core.Event import Event
from dHydra.config import connection as CON
from dHydra.config import const as C
from dHydra.core.Functions import *
from dHydra.core.Globals import *
# --- 导入自定义配置
from .connection import *
from .const import *
from .config import *
# 以上是自动生成的 #
from datetime import datetime
import time
import requests
import websockets
import getpass
import base64,rsa,binascii
import json
import asyncio
import threading
import functools
import re
import gc

class SinaLevel2WSProducer(Producer):
	def __init__(self, name = None, username = None, pwd = None, symbols = None, hq = 'hq_pjb', query = ['quotation', 'orders', 'deal', 'info'], **kwargs):
		super().__init__( name=name, **kwargs )
		if (username is None):
			if config["sinaUsername"] is not None:
				self.username = config["sinaUsername"]
			else:
				self.username = input('请输入新浪登录帐号：')
		else:
			self.username=username
		if (pwd is None):
			if config["sinaPassword"] is not None:
				self.pwd = config["sinaPassword"]
			else:
				self.pwd = getpass.getpass("输入登录密码（密码不会显示在屏幕上，输入后按回车确定）:")
		else:
			self.pwd = pwd
		self.rsaPubkey = '10001'
		self.ip = util.get_client_ip()
		self.session = requests.Session()
		self.hq = hq
		self.isLogin = self.login()
		self.query = query
		if symbols is None:
			sina = V('Sina')
			self.symbols = sina.get_symbols()
		else:
			self.symbols = symbols
		self.websockets = dict()

	def login(self):
		self.session.get("http://finance.sina.com.cn/realstock/company/sz300204/l2.shtml")
		su = base64.b64encode(self.username.encode('utf-8'))

		preLogin = self.session.get( URL_PRELOGIN, params = PARAM_PRELOGIN( su ), headers = HEADERS_LOGIN)
		preLogin = json.loads( preLogin.text[len("sinaSSOController.preloginCallBack("):-1] )

		sp = self.get_sp( self.pwd, preLogin["pubkey"], int(preLogin["servertime"]), preLogin["nonce"] )

		self.loginResponse = self.session.post(
			URL_SSOLOGIN
		,	params = PARAM_LOGIN()
		,	data = DATA_LOGIN(
				su = su
			,	servertime = int( preLogin["servertime"] )
			,	nonce = preLogin["nonce"]
			,	rsakv = preLogin["rsakv"]
			,	sp = sp
			)
		,	headers = HEADERS_LOGIN
		)
		if (self.loginResponse.json()["retcode"]=='0'):
			print( "登录成功: %s, uid = %s" % ( self.loginResponse.json()["nick"], self.loginResponse.json()["uid"]) )

			i = 0
			for url in self.loginResponse.json()["crossDomainUrlList"]:
				req = self.session.get( url,headers = HEADERS_CROSSDOMAIN( CROSSDOMAIN_HOST[i] ) )
				i += 1
			return True
		else:
			print( "Authentication Failed..." )
			print( self.loginResponse.json() )
			return False

	# RSA2 encoding
	def get_sp(self, passwd, pubkey, servertime, nonce):
		key = rsa.PublicKey(int(pubkey, 16), int('10001', 16))
		message = str(servertime) + '\t' + str(nonce) + '\n' + str(passwd)
		passwd = rsa.encrypt(message.encode('utf-8'), key)
		return binascii.b2a_hex(passwd).decode('ascii')

	@asyncio.coroutine
	def get_ws_token(self,qlist):
		loop = asyncio.get_event_loop()
		async_req = loop.run_in_executor(None, functools.partial( self.session.get, 
			URL_WSKT_TOKEN
		,	params 	=	PARAM_WSKT_TOKEN(ip=self.ip,qlist=qlist, hq = self.hq)
		,	headers =	HEADERS_WSKT_TOKEN()
		,	timeout =	5
		) )
		req = yield from async_req
		self.logger.info(req.text)
		response = re.findall(r'(\{.*\})',req.text)[0]
		response = json.loads( response.replace(',',',"').replace('{','{"').replace(':','":') )
		gc.collect()
		return response

	# 2cn_是3秒一条的Level2 10档行情
	# 2cn_symbol_0,2cn_symbol_1是逐笔数据
	# 2cn_symbol_orders是挂单数据
	# symbol_i是基本信息
	def generate_qlist(self,qlist,symbol):
		if 'quotation' in self.query:
			if qlist!='':
				qlist += ','
			qlist += "2cn_%s" % (symbol)
		if 'orders' in self.query:
			if qlist!='':
				qlist += ','
			qlist += "2cn_%s_orders" % (symbol)
		if 'deal' in self.query:
			if qlist!='':
				qlist += ','
			qlist += "2cn_%s_0,2cn_%s_1" % (symbol, symbol)
		if 'info' in self.query:
			if qlist!='':
				qlist += ','
			qlist += "%s_i" % (symbol)		
		return qlist

	@asyncio.coroutine
	def create_ws(self, qlist, symbolList ):
		retry = True
		while retry:
			try:
				response = yield from self.get_ws_token(qlist)
				if response["msg_code"] == 1:
					token = response["result"]
					self.logger.info("成功获取到token, symbolList = {}".format(symbolList) )
					retry = False
				else:
					self.logger.info(response["result"])
			except Exception as e:
				self.logger.warning(e)

		url_wss = 'wss://ff.sinajs.cn/wskt?token=' + token + '&list=' + qlist

		while True:	# 建立websocket连接
			try:
				ws = yield from websockets.connect(url_wss)
				self.websockets[ symbolList[0] ] = dict()
				self.websockets[ symbolList[0] ]["ws"] = ws
				self.websockets[ symbolList[0] ]["qlist"] = qlist
				self.websockets[ symbolList[0] ]["token"] = token
				self.websockets[ symbolList[0] ]["renewed"] = datetime.now()
				self.websockets[ symbolList[0] ]["trialTime"] = 0
				self.logger.info("成功建立ws连接, {}, symbolList = {}".format(threading.current_thread().name, symbolList))
				break
			except Exception as e:
				self.logger.warning("重试 websockets.connect , {}, symbolList = {}".format(threading.current_thread().name, symbolList) )

		# gc.collect()

		while self._active:
			try:
				message = yield from ws.recv()
				event = Event(eventType = 'SinaLevel2WS', data = message)

				for q in self._subscriber:
					q.put(event)

			except Exception as e:
				self.logger.error("{},{}".format(e, threading.current_thread().name) )
				ws.close()
				yield from self.create_ws(qlist = qlist, symbolList = symbolList)

	@asyncio.coroutine
	def renew_token(self, symbol):
		try:
			response = yield from self.get_ws_token( self.websockets[ symbol ]["qlist"] )
			if response["msg_code"] == 1:
				token = response["result"]
				self.websockets[ symbol ]["token"] = token
				self.websockets[ symbol ]["renewed"] = datetime.now()
				yield from self.websockets[ symbol ]["ws"].send("*"+token)
				self.websockets[ symbol ]["trialTime"] = 0
			else:
				self.websockets[ symbol ]["trialTime"] += 1
				self.logger.info(response["result"])
		except Exception as e:
			self.websockets[ symbol ]["trialTime"] += 1
			self.logger.warning("token获取失败第{}次，待会儿重试".format( self.websockets[ symbol ]["trialTime"] ))
		gc.collect()


	def websocket_creator(self):
		loop = asyncio.new_event_loop()
		asyncio.set_event_loop(loop)
		# 首先从新浪获取股票列表
		symbolList = self.symbols
		# Cut symbolList
		weight = (len(self.query)+1) if ('deal' in self.query) else len(self.query)
		step = int(64/weight)
		symbolListSlice = [symbolList[ i : i + step] for i in range(0, len(symbolList), step)]

		tasks = list()
		for symbolList in symbolListSlice:
			qlist = ''
			for symbol in symbolList:
				qlist = self.generate_qlist(qlist=qlist,symbol=symbol)
			qlist = qlist.lower()
			tasks.append( self.create_ws(qlist,symbolList = symbolList) )

		loop.run_until_complete( asyncio.wait(tasks) )
		loop.close()

	# 用于定时发送空字符串
	def token_sender(self):
		while True:
			self.logger.info("开启话唠模式每55秒的定时与服务器聊天")
			start = datetime.now()
			tasks = list()
			loop = asyncio.new_event_loop()
			asyncio.set_event_loop(loop)

			for symbol in self.websockets.keys():
				ws = self.websockets[ symbol ]["ws"]
				if ws.open:
					tasks.append( ws.send("*"+self.websockets[symbol]["token"]) )

			if len(tasks)>0:
				loop.run_until_complete( asyncio.wait(tasks) )
				loop.close()
			self.logger.info("消息全部发送完毕. 耗时：%s" % (datetime.now()-start).total_seconds() )
			time.sleep(55)

	# 持续检查一次更新token
	def token_renewer(self):
		while True:
			loop = asyncio.new_event_loop()
			asyncio.set_event_loop(loop)
			tasks = list()
			for symbol in self.websockets.keys():
				ws = self.websockets[ symbol ]["ws"]
				if ws.open:
					if (datetime.now()-self.websockets[ symbol ]["renewed"]).total_seconds()>180:
						tasks.append( self.renew_token( symbol ) )

			if len(tasks)>0:
				loop.run_until_complete( asyncio.wait(tasks) )
				loop.close()
			time.sleep(1)
			gc.collect()


	def handler(self):
		# 开启token manager
		tokenRenewer = threading.Thread( target = self.token_renewer )
		tokenSender = threading.Thread( target = self.token_sender )

		# creatorLoop = asyncio.new_event_loop()
		websocketCreator = threading.Thread( target = self.websocket_creator )

		tokenRenewer.start()		# 用于更新token
		tokenSender.start()			# 用于定时发送token
		websocketCreator.start()	# 用于建立websocket并接收消息

		tokenRenewer.join()
		tokenSender.join()
		websocketCreator.join()


# -*- coding: utf8 -*-
"""
# Created on 2016/04/12
# @author: Emptyset
# @contact: Emptyset110@gmail.com
"""
# 以下是自动生成的 #
# --- 导入系统配置
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
import dHydra.core.util as util
from dHydra.core.Producer import Producer
from dHydra.core.Event import Event
from dHydra.config import connection as CON
from dHydra.config import const as C
from dHydra.core.Functions import *
# --- 导入自定义配置

# 以上是自动生成的 #
import threading
import time
import asyncio
import functools
from datetime import datetime

class SinaFreeQuoteProducer(Producer):
	def __init__(self, name = None, symbols = None ,**kwargs):
		super().__init__( name=name, **kwargs )
		self.sina = V('Sina')
		if symbols is None:
			self.symbols = self.sina.get_symbols()
		else:
			self.symbols = symbols
		[self.szSymbols, self.shSymbols] = util.split_symbols(self.symbols)
		self.szTime = datetime(1970,1,1)
		self.shTime = datetime(1970,1,1)

	# 定时器触发的函数
	def thread_target(self, loop):
		asyncio.set_event_loop(loop)
		retry = True
		update = True
		sz = None
		sh = None
		while retry:
			try:
				[ szTime, shTime ] = self.sina.get_ticktime()
				if (szTime > self.szTime) & (len(self.szSymbols)>0):
					self.szTime = szTime
					szUpdate = True
				else:
					szUpdate = False

				if (shTime > self.shTime) & (len(self.shSymbols)>0):
					self.shTime = shTime
					shUpdate = True
				else:
					shUpdate = False
				if szUpdate and shUpdate:
					[ sz ,sh ] = self.sina.get_realtime_quotes( symbols = self.symbols , dataframe=True, loop = loop, split = True )
				elif szUpdate:
					sz = self.sina.get_realtime_quotes( symbols = self.szSymbols , dataframe=True, loop = loop )
				elif shUpdate:
					sh = self.sina.get_realtime_quotes( symbols = self.shSymbols , dataframe=True, loop = loop )
				retry = False
			except Exception as e:
				print(e)
		# print( time.time() - start )
		if sz is not None:
			eventSZ = Event( eventType = 'SinaFreeQuote', data = sz )
			eventSZ.time = szTime
			eventSZ.localtime = time.time()
			eventSZ.exchange = 'SZ'
			for q in self._subscriber:
				q.put(eventSZ)
		if sh is not None:
			eventSH = Event( eventType = 'SinaFreeQuote', data = sh )
			eventSH.time = shTime
			eventSH.localtime = time.time()
			eventSH.exchange = 'SH'
			for q in self._subscriber:
				q.put(eventSH)

	def handler(self):
		while self._active:
			loop = asyncio.new_event_loop()
			t = threading.Thread( target = functools.partial( self.thread_target, loop ) )
			t.start()
			time.sleep(1)



# -*- coding: utf8 -*-
"""
工具类
Created on 03/01/2016
@author: Wen Gu
@contact: emptyset110@gmail.com
"""
import dHydra.config.const as C
import requests
import threading
import asyncio
import math
import time
from datetime import datetime
import pandas
import os
import ntplib
from pandas import DataFrame
from pymongo import MongoClient
import re
import random

def _code_to_symbol(code):
	"""
		生成symbol代码标志
		@author: Jimmy Liu
		@group : waditu
		@contact: jimmysoa@sina.cn
	"""
	if code in C.INDEX_LABELS:
		return C.INDEX_LIST[code]
	else:
		if len(code) != 6 :
			return ''
		else:
			return 'sh%s'%code if code[:1] in ['5', '6', '9'] else 'sz%s'%code

def symbol_list_to_code(symbolList):
	codeList = []
	for symbol in symbolList:
		codeList.append(symbol[2:8])
	return codeList

def _get_public_ip():
	return requests.get('http://ipinfo.io/ip').text.strip()

def get_client_ip():
	response = requests.get( 'https://ff.sinajs.cn/?_=%s&list=sys_clientip' % int(time.time()*1000) ).text
	ip = re.findall(r'\"(.*)\"', response)
	return ip[0]

# 用于将一个loop交给一个线程来完成一些任务
def thread_loop(loop, tasks):
	# loop = asyncio.new_event_loop()
	asyncio.set_event_loop(loop)
	loop.run_until_complete( asyncio.wait(tasks) )
	loop.close()

# 用于将一个list按一定步长切片，返回这个list切分后的list
def slice_list(step = None, num = None, dataList=None):
	if not ( (step is None) & (num is None) ):
		if num is not None:
			step = math.ceil(len(dataList)/num)
		return [dataList[ i : i + step] for i in range(0, len(dataList), step)]
	else:
		print("step和num不能同时为空")
		return False

# n个任务交给m个Thread完成
def threads_for_tasks(taskList):
	threads = []
	for task in taskList:
		t = threading.Thread(target = task.target, args = task.args)
		threads.append(t)
	for t in threads:
		t.start()
		print("开启线程：",t.name)
	for t in threads:
		t.join()

def symbols_to_string(symbols):
	if isinstance(symbols, list) or isinstance(symbols, set) or isinstance(symbols, tuple) or isinstance(symbols, pandas.Series):
		return ','.join(symbols)
	else:
		return symbols

def datetime_to_timestamp(dt, timeFormat = 'ms'):
	if timeFormat == 'ms':
		return int( time.mktime( dt.timetuple() )*1000 )
	elif timeFormat == 's':
		return int( time.mktime( dt.timetuple() ) )

def date_to_timestamp(date, dateFormat = '%Y-%m-%d', timeFormat = 'ms'):
	return datetime_to_timestamp( dt = datetime.strptime(date, dateFormat) , timeFormat = timeFormat)

def string_to_date(date):
	return datetime.strptime(date, "%Y-%m-%d").date()

def timestamp_to_datetime(timestamp, timeFormat='ms'):
	if timeFormat == 'ms':
		timestamp = timestamp/1000
	return datetime.strftime(timestamp)

def time_now():
	return int(time.time()*1000)

# 从国家授时中心获取时间戳
def get_network_time():
	start = time.time()
	c = ntplib.NTPClient() 
	response = c.request('pool.ntp.org') 
	ts = response.tx_time 
	return ts

def check_time(precision = 0.1):
	duration = 2.0
	while duration > precision:
		try:
			print("{}, 开始获取网络时间戳".format( time.time() ))
			start = time.time()
			networkTime = get_network_time()
			end = time.time()
			duration = end-start
		except Exception as e:
			print("获取网络时间出了点小状况，正重试", duration)
	# print("网络耗时：{}".format( duration ) )
	# print("{}, 网络时间戳".format( networkTime ) )
	# print("{}, 现在时间戳".format( time.time()) )
	difference = networkTime - (start+duration)
	print("difference = {}, (本地时间戳+difference)=网络时间戳".format(difference))
	return difference

def split_symbols(symbols):
	df = DataFrame(symbols, columns = ['s'])
	sz = list(df[ df.s > 'sz' ]["s"])
	sh = list(df[ df.s < 'sz' ]["s"])
	return [ sz, sh ]

def upper(dataList):
	for i in range( 0, len(dataList) ):
		dataList[i] = dataList[i].upper()
	return dataList



# -*- coding: utf8 -*-
"""
Connection Settings
Created on 02/26/2016
@description:	Used for 
@author: 		Wen Gu
@contact: 		emptyset110@gmail.com
"""

"""
雪球
"""
HEADERS_XUEQIU = {
	"Accept"			:	"application/json, text/javascript, */*; q=0.01"
,	"Accept-Encoding"	:	"gzip, deflate, sdch"
,	"Accept-Language"	:	"en-US,en;q=0.8"
,	"cache-control"		:	"no-cache"
,	"Connection"		:	"keep-alive"
,	"Host"				:	"xueqiu.com"
,	"Referer"			:	"http://xueqiu.com/hq"
,	"User-Agent"		:	"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/49.0.2623.110 Safari/537.36"
,	"X-Requested-With"	:	"XMLHttpRequest"
}

CONST_XUEQIU_QUOTE_ORDER_COLUMN = "symbol,name,current,chg,percent,last_close,open,high,low,volume,amount,market_capital,pe_ttm,high52w,low52w,hasexist"
CONST_XUEQIU_QUOTE_COLUMN = "symbol,exchange,code,name,current,percentage,change,open,high,low,close,last_close,high52week,low52week,volume,volumeAverage,marketCapital,eps,pe_ttm,pe_lyr_beta,totalShares,time,afterHours,afterHoursPct,afterHoursChg,updateAt,dividednd,yield,turnover_rate,instOwn,rise_stop,fall_stop,currency_unit,amount,net_assets,hasexist,has_warrant,type,flag,rest_day,amplitude,lot_size,tick_size,kzz_stock_symbol,kzz_stock_name,kzz_stock_current,kzz_convert_price,kzz_convert_value"
URL_XUEQIU_HQ = "https://xueqiu.com/hq"
URL_XUEQIU_QUOTE_ORDER = lambda page,columns,stockType : "http://xueqiu.com/stock/quote_order.json?page=%s&size=90&order=desc&exchange=CN&stockType=%s&column=%s&orderBy=symbol&_=%s"	% ( page,stockType, columns,int(time.time()*1000) )
URL_XUEQIU_QUOTE = lambda symbol : "http://xueqiu.com/v4/stock/quote.json?code=%s&_=%s" % (symbol, int(time.time()*1000) )
URL_XUEQIU_PANKOU = lambda symbol : "http://xueqiu.com/stock/pankou.json?symbol=%s&_=%s" % (symbol, int(time.time()*1000) )
#"{"symbol":"SZ300061","time":"Mar 18, 2016 10:38:12 AM","bp1":21.4,"bc1":15112,"bp2":21.36,"bc2":20,"bp3":21.35,"bc3":122,"bp4":21.34,"bc4":1,"bp5":21.33,"bc5":1,"bp6":0.0,"bc6":0,"bp7":0.0,"bc7":0,"bp8":0.0,"bc8":0,"bp9":0.0,"bc9":0,"bp10":0.0,"bc10":0,"current":21.4,"sp1":0.0,"sc1":0,"sp2":0.0,"sc2":0,"sp3":0.0,"sc3":0,"sp4":0.0,"sc4":0,"sp5":0.0,"sc5":0,"sp6":0.0,"sc6":0,"sp7":0.0,"sc7":0,"sp8":0.0,"sc8":0,"sp9":0.0,"sc9":0,"sp10":0.0,"sc10":0,"buypct":100.0,"sellpct":0.0,"diff":15256,"ratio":100.0}"

"""
tushare
"""
URL_TUSHARE_BASICS = "http://218.244.146.57/static/all.csv"


# -*- coding: utf8 -*-
"""
Action类
每个Action自带一个Queue
Created on 03/30/2016
@author: Wen Gu
@contact: emptyset110@gmail.com
"""
import multiprocessing
import time
import threading
import logging
from dHydra.core.Globals import *
from dHydra.core.Functions import *
from dHydra.app import PRODUCER_NAME, PRODUCER_HASH
from abc import ABCMeta

class Action(threading.Thread):
	__metaclass__ = ABCMeta
	def __init__(self, name, **kwargs):
		super().__init__()
		self.logger = self.get_logger()
		self._name = name
		self._kwargs = kwargs
		self._queue = multiprocessing.Queue()
		self._producers = set()

		# 在子类中被重写
		# self._producerList = []

		# _running决定action是否开启“定时处理”
		# _active决定action是否会执行数据处理过程
		# -----------------------------------
		# 二者既不充分也不必要：
		# 	_running==False & _active==True		: "事件触发"状态
		#	_running==True & _active==True		: "定时触发"状态
		#	_running==True & _active==False		: 定时线程开启但是暂停处理数据
		#	_running==False & _active==False	: Action完全停止
		self._active = False
		self._running = False
		self._auto_load_producers()

	def get_logger(self):
		logger = logging.getLogger(self.__class__.__name__)
		return logger

	# action订阅producer
	def _subscribe(self, instance):
		instance._add_subscriber(queue = self._queue)

	# action取消订阅producer
	def _unsubscribe(self, instance):
		instance._remove_subscriber(queue = self._queue)

	def _activate(self):
		print('[激活Action]\t', self._name)
		self._active = True

	def _deactivate(self):
		self._active = False
		for producer in list(self._producers):
			producer._remove_subscriber(self._queue)
		print('[暂停Action]\t', self._name)

	# 运行action即相当于采用“定时处理”的模式
	def run(self):
		self._auto_start_producers()
		self._running = True
		self._activate()
		print('[开启Action]\t', self._name)
		while self._running:
			if self._active:
				t = threading.Thread( target=self.handler )
				t.start()
				# t.join()
			time.sleep(self._interval)
		self._end()

	def is_active(self):
		return self._active

	def is_running(self):
		return self._running

	# 通知action进行一次数据处理，这是“事件触发”的模式
	def _notify(self):
		self._handler()

	def _end(self):
		for producer in list(self._producers):
			producer._remove_subscriber(self._queue)
		print('[结束Action]\t', self._name)

	def _stop(self):
		self._running = False

	# action根据self._producerList来自动加载producers
	def _auto_load_producers(self):
		for kwargs in self._producerList:
			instance = P(**kwargs, **self._kwargs)
			self._producers.add( instance )
			self._subscribe(instance)

	def _auto_start_producers(self):
		for producer in list(self._producers):
			if not producer.is_running():
				producer.start()

	# 需要在子类中重写的数据处理方法
	def handler(self):
		pass


# -*- coding: utf8 -*-
"""
Config
Created on 02/22/2016
@description:	Used for 
@author: 		Wen Gu
@contact: 		emptyset110@gmail.com
"""
VERSION = '0.2.0'

#	路径配置		
#	Path Configuration
PATH_DATA_ROOT = './data/'
PATH_DATA_REALTIME = 'stock_realtime/'
PATH_DATA_L2 = 'stock_l2/'

#	Mongodb连接配置
#	Mongodb Configuration




# -*- coding: utf8 -*-
"""
主控制程序入口
"""
import traceback
import sys
import os

# print("加载：app.py")

from dHydra.core.Globals import *
from dHydra.core.Functions import *

def start_action(actionList, actionArgs):
	for action in actionList:
		if action in actionArgs.keys():
			print(actionArgs[action])
			actionInstance = A(action, **actionArgs[action])
		else:
			actionInstance = A(action)
		actionDict[action] = actionInstance
	for action in actionList:
		actionDict[action].start()
	for action in actionList:
		actionDict[action].join()


# -*- coding:utf-8 -*-
"""
dHydra电影数据接口接口：
Created on 03/08/2016
@author: Wen Gu
@contact: emptyset110@gmail.com
"""
import requests
from . import connection as CON
from pandas import DataFrame as df
import pandas
import re

"""
猫眼电影数据
"""
class Maoyan:
	def __init__(self):
		self.session = requests.Session()
		self.get_cities()
		pass

	"""
	初始化获取城市表
	"""
	def get_cities(self):
		self.city = self.session.get("http://pf.maoyan.com/getAllCity.json").json()
		self.allCities = list()
		for key in self.city["allCity"].keys():
			self.allCities += self.city["allCity"][key]
		self.allCities = pandas.DataFrame(self.allCities).set_index("id")
		self.cityTier = self.city["cityTier"]
		self.hotCities = self.city["hotCity"]

	"""
	按日期获取电影票房数据
	"""
	def get_box(self, date):
		self.box = self.session.get(CON.URL_MAOYAN_DATE, params = CON.PARAM_DATE(date) ,headers = CON.HEADERS_MAOYAN)
		return self.box

	"""
	按日期获取电影票房数据，返回dataframe
	"""
	def get_box_dataframe(self, date):
		box_dict = self.get_box(date)
		box_df = df.from_dict(box_dict["data"])
		return box_df

	"""
	按日期获取电影票房数据并导出到csv文件
	"""
	def export_box_csv(self, date):
		box_df = self.get_box_dataframe(date)
		box_df.to_csv( '%s.csv'% date )

	"""
	按电影id获取电影票房数据
	"""
	def get_movie(self, movie):
		movie = self.session.get(CON.URL_MAOYAN_MOVIE, params = CON.PARAM_MOVIEID(movie), headers = CON.HEADERS_MAOYAN)
		return movie.json()

	def get_movie_dataframe(self, movie):
		movie_dict = self.get_movie(movie)
		movie_df = df.from_dict(movie_dict["data"])
		return movie_df

	def export_movie_csv(self, movie):
		movie_dict = self.get_movie(movie)
		df.from_dict(movie_dict).to_csv( "%s-%s-%s.csv" % (movie_dict["releaseDate"], movie_dict["movieName"],str(movie) ) )

	def get_movie_basics(self, movie):
		return self.get_movie(movie)["movieName"]


	"""
	通过电影名/导演/演员名字获取电影id
	Arguments:
	---
		name:	查询关键词
	---
	返回:	默认返回dict，如果设置dataframe=True，则返回DataFrame
	---
		movieid
	"""
	def search_movie_id(self, name):
		response = self.session.get("http://pf.maoyan.com/search?_v_=yes&key=%s&page=1&size=1000"%name).text
		movieid = re.findall(r'<article class=\"indentInner canTouch\" data-url=\"\/movie\/(.*)\">',response)
		return movieid


	"""
	按电影获取它受众比例
	---
	arguments:
		---
		movie:	电影id(可以为空)
		name:	电影名/导演名/演员名字
	---
	return:
		---
		womanRate		:	女性比例
		manRate			:	男性比例
		age_11_15_Rate	:	11-15岁
		age_16_20_Rate	:	16-20岁
		age_21_25_Rate	:	21-25岁
		age_26_30_Rate	:	26-30岁
		age_31_35_Rate	:	31-35岁
		age_36_40_Rate	:	36-40岁
		age_41_50_Rate	:	41-50岁
		updateTime		:	数据更新时间
	---
	e.g.:
		movie.audience_distribution(name="夏洛特烦恼",dataframe=True)
		---
			    womanRate age_11_15_Rate age_16_20_Rate manRate age_31_35_Rate  \
		movieId                                                                  
		246082       57.9            0.3           7.65    42.1          13.62   

		        age_26_30_Rate age_41_50_Rate age_36_40_Rate            updateTime  \
		movieId                                                                      
		246082            33.4           2.67           5.82  2016年03月09日 01:05:41   

		        age_21_25_Rate  
		movieId                 
		246082           36.54
	"""
	def audience_distribution(self, movie=None, name=None, dataframe=False):
		if movie == None:
			movieid = self.search_movie_id(name)
			if len(movieid)==0:
				print("没有填入电影id，且没有查询到与\"%s\"有关的电影"%name)
				return None

		if dataframe:
			result = None
		else:
			result = list()

		for movie in movieid:
			data = self.session.get(
				"http://pf.maoyan.com/movie/viewerStatistic.json?movieId=%s"%movie
			,	headers = CON.HEADERS_MAOYAN
			).json()["data"]
			if data == "":
				continue

			if dataframe:
				data_df = pandas.DataFrame(data, index = [data["movieId"]])
				if (result == None):
					result = data_df
				else:
					result = result.append(data_df)
			else:
				result.append(data)
		print("共找到%d个与\"%s\"相关的电影"%(len(movieid),name))
		print("共找到%d个与\"%s\"相关的电影数据"%(len(movieid),name))
		return result

	"""
	想看指数：按城市，按日期两种排序方式
	"""
	def wish(self,movie=None,name=None,wish_type="date", dataframe=False):
		if movie == None:
			movieid = self.search_movie_id(name)
			if len(movieid)==0:
				print("没有填入电影id，且没有查询到与\"%s\"有关的电影"%name)
				return None

		if dataframe:
			result = None
		else:
			result = list()

		# if (wish_type=="date"):
		# 	data = self.session.get()
		# elif (wish_type=="city"):
		# 	


# -*- coding: utf8 -*-
# import dHydra
import codecs
import os
import sys
 
try:
    from setuptools import setup
except:
    from distutils.core import setup
"""
打包的用的setup必须引入，
"""

def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()
 
NAME = "dHydra"
"""
名字，一般放你包的名字即可
"""
 
PACKAGES = [	"dHydra"
			,	"dHydra.core"
			,	"dHydra.producer"
			,	"dHydra.producer.SinaLevel2WS"
			,	"dHydra.producer.SinaFreeQuote"
			,	"dHydra.vendor"
			,	"dHydra.vendor.Maoyan"
			,	"dHydra.vendor.Xueqiu"
			,	"dHydra.vendor.Sina"
			,	"dHydra.action"
			,	"dHydra.action.SinaTickToMongo"
			,	"dHydra.action.PrintSinaL2"
			,	"dHydra.action.Demo"
			,	"dHydra.action.SinaL2TCP"
			,	"dHydra.config"
			]

DESCRIPTION = "A framework for saving & data mining Chinese Stocks"
 
LONG_DESCRIPTION = ""
"""
参见read方法说明
"""

KEYWORDS = ("Finance","Chinese Stock","Quant","Framework")
"""
关于当前包的一些关键字，方便PyPI进行分类。
"""
 
AUTHOR = "Wen Gu"
 
AUTHOR_EMAIL = "emptyset110@gmail.com"
 
URL = "http://dHydra.org"

VERSION = "0.10.4"

LICENSE = "Apache Software License"
 
setup(
    name = NAME
,   version = VERSION
,   description = DESCRIPTION
,   long_description = LONG_DESCRIPTION
,   classifiers = [
        'License :: OSI Approved :: Apache Software License'
    ,   'Programming Language :: Python'
    ,   'Intended Audience :: Financial and Insurance Industry'
    ,   'Operating System :: OS Independent'
    ]
,   install_requires = ["requests","numpy","pandas","lxml","tushare","pymongo","websockets","rsa","ntplib"]
,   keywords = KEYWORDS
,   author = AUTHOR
,   author_email = AUTHOR_EMAIL
,   url = URL
,   license = LICENSE
,   packages = PACKAGES
,   include_package_data=True
,   zip_safe=True
)


# -*- coding: utf8 -*-
import time
# 高速行情接口
URL_QUOTATION = lambda symbols: "http://hq.sinajs.cn/?rn=%s&list=%s" % ( int( time.time() )*1000, symbols )

SINA_QUOTE_COLUMNS = [	'name', 'open'
					,	'pre_close', 'price', 'high', 'low', 'bid', 'ask', 'volume', 'amount'
					,	'b1_v',	'b1_p', 'b2_v', 'b2_p', 'b3_v', 'b3_p', 'b4_v', 'b4_p', 'b5_v', 'b5_p'
					,	'a1_v',	'a1_p', 'a2_v', 'a2_p', 'a3_v', 'a3_p', 'a4_v', 'a4_p', 'a5_v', 'a5_p'
					,	'date', 'time', 'ms']

SINA_QUOTE_COLUMNS_2 = [	'name', 'open'
					,	'pre_close', 'price', 'high', 'low', 'bid', 'ask', 'volume', 'amount'
					,	'b1_v',	'b1_p', 'b2_v', 'b2_p', 'b3_v', 'b3_p', 'b4_v', 'b4_p', 'b5_v', 'b5_p'
					,	'a1_v',	'a1_p', 'a2_v', 'a2_p', 'a3_v', 'a3_p', 'a4_v', 'a4_p', 'a5_v', 'a5_p'
					,	'date', 'time', 'ms', 'symbol']

DATA_LOGIN = lambda su,servertime,nonce,rsakv,sp: {
	"entry"			:	"finance"
,	"gateway"		:	1
,	"from"			:	""
,	"savestate" 	:	30
,	"useticket" 	:	0
,	"pagerefer" 	:	""
,	"vsnf"			:	"1"
,	"su"			:	su
,	"service"		:	"sso"
,	"servertime"	:	servertime
,	"nonce"			:	nonce
,	"pwencode"		:	"rsa2"
,	"rsakv"			:	rsakv
,	"sp"			:	sp
,	"sr" 			:	"1920*1080"
,	"encoding"		:	"UTF-8"
,	"cdult"			:	3
,	"domain"		:	"sina.com.cn"
,	"prelt"			:	72
,	"returntype"	:	"TEXT"
}

URL_CROSSDOMAIN = "http://login.sina.com.cn/sso/crossdomain.php"
URL_SSOLOGIN = "http://login.sina.com.cn/sso/login.php"
URL_SSOLOGOUT = "http://login.sina.com.cn/sso/logout.php"
URL_UPDATECOOKIE = "http://login.sina.com.cn/sso/updatetgt.php"
URL_PRELOGIN = "http://login.sina.com.cn/sso/prelogin.php"
URL_PINCODE = "http://login.sina.com.cn/cgi/pin.php"
URL_VFVALID = "http://weibo.com/sguide/vdun.php"
PARAM_LOGIN = lambda : {
	"client" 		:	CLIENT
,	"_"				:	int(time.time()*1000)
}
PARAM_PRELOGIN = lambda su : {
	"entry"			:	"finance"
,	"callback"		:	"sinaSSOController.preloginCallBack"
,	"su"			:	su
,	"rsakt"			:	'mod'
,	"client"		:	CLIENT
,	"_"				:	int(time.time()*1000)
}
PARAM_L2HIST = lambda symbol,page,stime,etime: {
	"symbol"			:	symbol
,	"callback"			:	"jQuery17209838373235483986_" + str( int(time.time()*1000) )
,	"pageNum"			:	10000
,	"page"				:	page
,	"stime"				:	stime
,	"etime"				:	etime
,	"sign"				:	''
,	"num"				:	'20'
,	"_"					:	int(time.time()*1000)
}
HEADERS_LOGIN = {
	"Accept" : '*/*'
,	"Accept-Encoding" : 'gzip, deflate, sdch'
,	"Accept-Language" : 'en-US,en;q=0.8'
,	"Connection" : 'keep-alive'
,	"Host" : 'login.sina.com.cn'
,	"Referer" : 'http://finance.sina.com.cn/'
,	"User-Agent" : 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 Safari/537.36'
}
HEADERS_L2 = lambda symbol	: {
	'Host'				:	'stock.finance.sina.com.cn'
,	'Connection'		:	'keep-alive'
,	'Accept'			:	'*/*'
,	'User-Agent'		:	'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 Safari/537.36'
,	'Referer'			:	'http://vip.stock.finance.sina.com.cn/quotes_service/view/l2_tradedetail.php?symbol=%s' % symbol
,	'Accept-Encoding'	:	'gzip, deflate, sdch'
,	'Accept-Language'	:	'en-US,en;q=0.8'
}
HEADERS_CROSSDOMAIN = lambda host	:	{
	'Host'							:	host
,	'User-Agent'					:	'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/48.0.2564.116 Safari/537.36'
,	'Accept'						: 	'*/*'
,	'Accept-Language'				:	'en-US,en;q=0.5'
,	'Accept-Encoding'				:	'gzip, deflate'
,	'Referer'						:	'http://finance.sina.com.cn/realstock/company/sz300204/l2.shtml'
,	'Connection'					:	'keep-alive'
}
CLIENT = "ssologin.js(v1.4.18)"
CROSSDOMAIN_HOST = [
	"passport.weibo.com"
,	"crosdom.weicaifu.com"
,	"passport.weibo.cn"
]
URL_L2HIST = 'http://stock.finance.sina.com.cn/stock/api/openapi.php/StockLevel2Service.getTransactionList'
# 获取
URL_API_MARKET_CENTER_GETHQNODEDATA = lambda node: "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/Market_Center.getHQNodeData?num=5000&sort=symbol&asc=0&node=%s&symbol=&_s_r_a=page&page=1" % node
# node = hs_a, hs_b
# 数据格式中key缺少双引号
# [	
# 	{
# 		symbol:"sh600006"
# 	,	code:"600006"
# 	,	name:"东风汽车"
# 	,	trade:"6.210"
# 	,	pricechange:"0.000"
# 	,	changepercent:"0.000"
# 	,	buy:"0.000"
# 	,	sell:"0.000"
# 	,	settlement:"6.210"
# 	,	open:"0.000"
# 	,	high:"0.000"
# 	,	low:"0.000"
# 	,	volume:0
# 	,	amount:0
# 	,	ticktime:"10:12:31"
# 	,	per:88.336
# 	,	pb:1.94
# 	,	mktcap:1242000
# 	,	nmc:1242000
# 	,	turnoverratio:0
# 	}
# ]